@@ -37,10 +37,13 @@ class BaseDataset(metaclass=abc.ABCMeta):
                  stage: str = "train",
                  **kwargs):
         self.data_paths = data_paths or []
+        if not isinstance(self.data_paths, list):
+            raise ValueError('data_paths must be a list of string paths')
         self.augmentations = augmentations  # apply augmentation
-        self.cache = cache  # whether to cache WHOLE transformed dataset to memory
+        self.cache = cache  # whether to cache transformed dataset to memory
         self.shuffle = shuffle  # whether to shuffle tf.data.Dataset
-        if buffer_size <= 0 and shuffle: raise ValueError("buffer_size must be positive when shuffle is on")
+        if buffer_size <= 0 and shuffle:
+            raise ValueError("buffer_size must be positive when shuffle is on")
         self.buffer_size = buffer_size  # shuffle buffer size
         self.stage = stage  # for defining tfrecords files
         self.use_tf = use_tf
